#include <cassert>
#include <cstdlib>
#include <cstdio>

#include "board.h"


// this global variable is set in main.cpp and is adjustable from the command line
// (you are not allowed to make your own global variables)
extern int GLOBAL_TILE_SIZE;


// ==========================================================================
// CONSTRUCTOR
Board::Board(int i, int j) {
  board = std::vector<std::vector<Tile*> >
    ( (unsigned int)i,
      std::vector<Tile*>((unsigned int)j,NULL) );
}


// ==========================================================================
// ACCESSORS
Tile* Board::getTile(int i, int j) const {
  assert (i >= 0 && i < numRows());
  assert (j >= 0 && j < numColumns());
  return board[i][j];
}


// ==========================================================================
// MODIFIERS
void Board::setTile(int i, int j, Tile* t) {
  assert (i >= 0 && i < numRows());
  assert (j >= 0 && j < numColumns());
  assert (t != NULL);
  assert (board[i][j] == NULL);
  board[i][j] = t;
}

void Board::removeTile(int i, int j) {
  assert (i >= 0 && i < numRows());
  assert (j >= 0 && j < numColumns());
  
  board[i][j] = NULL;
}


// ==========================================================================
// PRINTING
void Board::Print() const {
  for (int b = 0; b < numRows(); b++) {
    for (int i = 0; i < GLOBAL_TILE_SIZE; i++) {
      for (int j = 0; j < numColumns(); j++) {
        if (board[b][j] != NULL) {
          board[b][j]->printRow(std::cout,i);
        } else {
          std::cout << std::string(GLOBAL_TILE_SIZE,' ');
        }
      }
      std::cout << "\n";
    }
  }
  fflush(stdout);
}

// ==========================================================================
bool Board::same(const Board &b){
  //see if two boards look the same, including size
  if(this->numRows() != b.numRows())
    return false;

  if(this->numColumns() != b.numColumns())
    return false;

  for(int j = 0; j < this->numRows(); ++j){
    for(int i = 0; i < this->numColumns(); ++i){
      //see if one of the tiles is null but not the other
      if(this->getTile(j,i) == NULL)
        if(b.getTile(j,i) != NULL)
          return false;

      if(b.getTile(j,i) == NULL)
        if(this->getTile(j,i) != NULL)
          return false;
      //check north, south, east, and west as all the same
      if(this->getTile(j,i)->getNorth() != b.getTile(j,i)->getNorth())
        return false;
      if(this->getTile(j,i)->getEast() != b.getTile(j,i)->getEast())
        return false;
      if(this->getTile(j,i)->getSouth() != b.getTile(j,i)->getSouth())
        return false;
      if(this->getTile(j,i)->getWest() != b.getTile(j,i)->getWest())
        return false;
    }
  }

  return true;
}