#include "foo.h"
#include <cmath>

bool PlaceFirstTile(int height, int width, const std::vector<Tile*> tiles, std::vector<Location> &locations, bool AllSolutions, std::vector<std::vector<Location> > &solutions, bool Roatations){
	if(!Roatations){
		//go through every possible placement
		for(int i = 0; i < width; i++){
			for(int j = 0; j < height; j++){
				//see if the first tile fits in here
				if(AcceptableFirstTile(i,j,tiles[0],width,height)){

					//first tile fits in, place it into the locations and board
					Board board(height, width);
					board.setTile(j,i,tiles[0]);
					locations.push_back(Location(j,i,0));

					//check rest of tiles using recursive function
					if(AllSolutions){
						PlaceTile(board,tiles,locations,1,AllSolutions, solutions, Roatations);	
					}
					else{ //if we only want one solution, we stop when we reach the bottom
						if(PlaceTile(board, tiles, locations, 1, AllSolutions, solutions, Roatations)){
							return true;
						}
					}
					//once we go through all posibilieies with this combination of location and tile, remove them
					locations.pop_back();
					board.removeTile(j,i);
				}
			}
		}
	}
	else{ //same as above, only going through an extra time with 90 degree rotation
		  //need to rotate 90 for the case where the board is not a square and the tiles wont fit with one orientation
		for(int r = 0; r < 2; r++){
			for(int i = 0; i < width; i++){
				for(int j = 0; j < height; j++){
					//see if the first tile fits in here
					if(AcceptableFirstTile(i,j,tiles[0],width,height)){
						//first tile fits in, place it into the locations and board
						Board board(height, width);
						board.setTile(j,i,tiles[0]);
						locations.push_back(Location(j,i,90*r));

						//check rest of tiles using recursive function
						if(AllSolutions){
							PlaceTile(board,tiles,locations,1,AllSolutions, solutions, Roatations);	
						}
						else{
							if(PlaceTile(board, tiles, locations, 1, AllSolutions, solutions, Roatations)){
								return true;
							}
						}
					
						locations.pop_back();
						board.removeTile(j,i);
					}
				}
			}
			tiles[0]->rotate(90);
		}
	}
	return false;
}

bool PlaceTile(Board board, const std::vector<Tile*> tiles, std::vector<Location> &locations, int index, bool AllSolutions, std::vector<std::vector<Location> > &solutions, bool Roatations){
	if(index == tiles.size()){
		//reached the end
		if(NoLooseRoads(board,locations,tiles)){
			//if all solutions, check for duplicates
			if(AllSolutions){
				if(DuplicateSolutions(locations, solutions)){
					solutions.push_back(locations);
				}
				return false;
			}
			else{
				//not looking for every solution, so it prints the solution down here if it worked
				std::cout << "Solution: ";
				for(int i = 0; i < locations.size(); i++){
					std::cout << locations[i];
				}
				return true;
			}
			
		}
		else
			return false;
	}
	//go through every possibility
	if(!Roatations){
		for(int j = 0; j < board.numRows(); ++j){
			for(int i = 0; i < board.numColumns(); ++i){
				//we are at location i,j. check if it is a valid tile placement
				if(ValidPlacement(tiles[index], board, i, j)){
					//its a valid placement
					//add this into the board and locations, keep going on the loop unless you get back false
					locations.push_back(Location(j,i,0));
					board.setTile(j,i,tiles[index]);
					if(AllSolutions){
						PlaceTile(board,tiles,locations, index + 1,AllSolutions, solutions, Roatations);	
					}
					else{
						if(PlaceTile(board, tiles, locations, index + 1, AllSolutions, solutions, Roatations)){
							return true;
						}
					}
					locations.pop_back();
					board.removeTile(j,i);
				}
			}
		}
		return false;
	}
	else{
		for(int r = 0; r < 360; r+=90){
			for(int j = 0; j < board.numRows(); ++j){
				for(int i = 0; i < board.numColumns(); ++i){
					//we are at location i,j. check if it is a valid tile placement
					if(ValidPlacement(tiles[index], board, i, j)){
						//its a valid placement
						//add this into the board and locations, keep going on the loop unless you get back false
						locations.push_back(Location(j,i,r));
						board.setTile(j,i,tiles[index]);
						if(AllSolutions){
							PlaceTile(board,tiles,locations, index + 1,AllSolutions, solutions, Roatations);	
						}
						else{
							if(PlaceTile(board, tiles, locations, index + 1, AllSolutions, solutions, Roatations)){
								return true;
							}
						}
					
						locations.pop_back();
						board.removeTile(j,i);
					}
				}
			}
			tiles[index]->rotate(90);
		}
		return false;
	}
}

bool ValidPlacement(Tile* tile, Board board, int x, int y){
	//false if there is already a tile there
	if(board.getTile(y,x) != NULL)
		return false;

	int counter = 4;
	//check if north is okay
	if(y > 0){ //if it is along the top
		//see if there is a tile above it
		if(board.getTile(y-1, x) != NULL){
			//make sure that the two match
			if(board.getTile(y-1,x)->getSouth() != tile->getNorth())
				return false;
		}
		else
			--counter;
	}
	else{
		if(tile->getNorth() != "pasture")
			return false;
		--counter;
	}

	//check south
	if(y < board.numRows() - 1){
		if(board.getTile(y + 1, x) != NULL){
			if(board.getTile(y + 1, x)->getNorth() != tile->getSouth())
				return false;
		}
		else
			--counter;

	}
	else{
		if(tile->getSouth() != "pasture")
			return false;
		--counter;
	}

	//check west
	if(x > 0){
		if(board.getTile(y, x - 1) != NULL){
			if(board.getTile(y, x - 1)->getEast() != tile->getWest())
				return false;
		}
		else
			--counter;
	}
	else{
		if(tile->getWest() != "pasture")
			return false;
		--counter;
	}

	//check east
	if(x < board.numColumns() - 1){
		if(board.getTile(y, x + 1) != NULL){
			if(board.getTile(y, x + 1)->getWest() != tile->getEast())
				return false;
		}
		else
			--counter;
	}
	else{
		if(tile->getEast() != "pasture")
			return false;
		--counter;
	}

	//see if any of the adjacent tiles exsisted
	if(counter == 0)
		return false;

	return true;

}

bool AcceptableFirstTile(int x, int y, Tile* tile, int width, int height){
	//make sure that no roads or cities are pointing into the void
	if(x == 0 && tile->getWest() != "pasture")
		return false;
	if(x == width - 1 && tile->getEast() != "pasture")
		return false;
	if(y == 0 && tile->getNorth() != "pasture")
		return false;
	if(y == height - 1 && tile->getSouth() != "pasture")
		return false;

	return true;

}

bool NoLooseRoads(Board board, std::vector<Location> locations, std::vector<Tile*> tiles){
	//make sure that that all boarders are pastures
	if(locations.size() != tiles.size()){
		std::cerr << "ERROR: not all tiles placed\n";
		return false;
	}

	//check north
	for(int i = 0; i < locations.size(); ++i){
		//see if the north has a loose road or city
		if(locations[i].row == 0 || board.getTile(locations[i].row - 1, locations[i].column) == NULL){
			//north cannot be anything other than pasture
			if(board.getTile(locations[i].row, locations[i].column)->getNorth() != "pasture"){
				return false;
			}
		}

		//check south
		if(locations[i].row == board.numRows() - 1 || board.getTile(locations[i].row + 1, locations[i].column) == NULL){
			//look to the south
			if(board.getTile(locations[i].row, locations[i].column)->getSouth() != "pasture"){
				//std::cout << "south failed for tile " << i + 1 << std::endl;
				return false;
			}
		}

		//check west
		if(locations[i].column == 0 || board.getTile(locations[i].row, locations[i].column - 1) == NULL){
			//look at west
			if(board.getTile(locations[i].row, locations[i].column)->getWest() != "pasture")
				return false;
		}

		//check east
		if(locations[i].column == board.numColumns() - 1 || board.getTile(locations[i].row, locations[i].column + 1) == NULL){
			if(board.getTile(locations[i].row, locations[i].column)->getEast() != "pasture")
				return false;
		}


	}

	return true;
}

bool DuplicateSolutions(std::vector<Location> solution, std::vector<std::vector<Location> > prevSolutions){
	//checks to see if the board is a translation of another one
	if(prevSolutions.size() == 0)
		return true;

	double distance;
	double calc;
	int counter = 0;

	for(int i = 0; i < prevSolutions.size(); ++i){
		//find the distance between solutions for the first location, if that matched the rest, it is a duplicate
		distance = sqrt( pow(solution[0].row - prevSolutions[i][0].row, 2) + pow(solution[0].column - prevSolutions[i][0].column, 2));

		bool finder = false;
		for(int k = 0; k < solution.size(); ++k){
			calc = sqrt( pow(solution[k].row - prevSolutions[i][k].row, 2) + pow(solution[k].column - prevSolutions[i][k].column, 2));
			if(calc != distance){
				finder = true;
				break;
			}
		}
		if(finder)
			++counter;
	}
	//if it matches none of them its true, optherwise false
	if(counter == prevSolutions.size())
		return true;
	return false;
}

